/* -*- Mode: java; tab-width: 8; c-basic-offset: 4 -*-
 * Copyright  1997, 1998 Netscape Communications Corporation, All Rights Reserved.
 */

package com.netscape.javascript.CORBA;

import com.netscape.javascript.*;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.portable.*;
import org.omg.CORBA.CompletionStatus;
import java.util.Hashtable;

/**
 * This class does marshalling of CORBA objects to the JS and back
 *
 * @author Ruslan Belkin
 */

public class   Marshal  {

    public  static Object extractObject (InputStream is, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        try {
            switch (tc.kind().value ())
            {
                case TCKind._tk_array   :
                case TCKind._tk_sequence:
                    return Array.extractObject (is, tc);

                case TCKind._tk_alias   :
                    return extractObject (is, tc.content_type());

                case TCKind._tk_except  :
                case TCKind._tk_struct  :
                    return Struct.extractObject(is, tc);

                default :
                    if (Simple.handledType (tc.kind().value ()))
                        return Simple.extractObject (is, tc);
            }
        }
        catch (org.omg.CORBA.UserException ue)   { /* should never happen */ }
        throw new org.omg.CORBA.MARSHAL ("cannot marshal type: " + tc, 0, CompletionStatus.COMPLETED_NO);
    }

    public static void copyObject (InputStream is, OutputStream os, TypeCode tc)
        throws org.omg.CORBA.MARSHAL
    {
        int tkind = tc.kind().value();

        try {
            switch (tkind)
            {
                case TCKind._tk_array   :
                case TCKind._tk_sequence:
                    Array.copyObject (is, os, tc);
                    break;
                case TCKind._tk_alias   :
                    copyObject (is, os, tc.content_type ());
                    break;
                case TCKind._tk_except  :
                case TCKind._tk_struct  :
                    Struct.copyObject(is, os, tc);
                    break;

                default:
                    if (Simple.handledType (tkind))
                        Simple.copyObject (is, os, tc);
                    else
                        throw new org.omg.CORBA.MARSHAL ("cannot marshal type: " + tc, 0, CompletionStatus.COMPLETED_NO);
            }
        }
        catch (org.omg.CORBA.UserException ue)   { /* should never happen */ }
    }

    public static void writeObject (OutputStream os, TypeCode tc, Object value)
        throws org.omg.CORBA.MARSHAL
    {
        int tkind  = tc.kind().value ();

        if (value instanceof InOut)
            value = ((InOut)value).value;
            
        try {
            switch (tkind)
            {
                case TCKind._tk_array   :
                case TCKind._tk_sequence:
                    Array.writeObject (os, tc, value);
                    break;
                case TCKind._tk_except  :
                case TCKind._tk_struct  :
                    Struct.writeObject(os, tc, value);
                    break;
                case TCKind._tk_alias   :
                    writeObject (os, tc.content_type (), value);
                    break;
                default:
                    if (Simple.handledType (tkind))
                        Simple.writeObject (os, tc, value);
                    else
                        throw new org.omg.CORBA.MARSHAL ("cannot cast value: " + value + " to type: " + tc, 0, CompletionStatus.COMPLETED_NO);
            }
        }
        catch (org.omg.CORBA.UserException ue)   { /* should never happen */ }
    }

    public static boolean handledType (int tkind)
    {
        return tkind == TCKind._tk_alias
            || Simple.handledType (tkind) || Struct.handledType (tkind) || Array.handledType (tkind);
    }

    public static Object convertFromAny (Any value)
    {
        int tkind = value.type ().kind().value();

        try {
            if (Struct.handledType (tkind))
                return new Struct (value);  // just a little optimization
            else
            if (Array.handledType (tkind))
                return new Array (value);

            return extractObject (value.create_input_stream (), value.type ());
        }
        catch (org.omg.CORBA.SystemException se)
        {
            Init.raise (new SystemException (se));
        }
        return null;
    }

    public static boolean convertIntoAny(Object value, Any any)
    {
        OutputStream os = any.create_output_stream ();

        try {
            writeObject (os, any.type (), value);
        }
        catch (org.omg.CORBA.SystemException se)
        {
            Init.raise (new SystemException (se));
            return false;
        }

        any.read_value (os.create_input_stream (), any.type ());
        return true;
    }

